from .clients import client


class _ClientMixin:
    _base_url = "https://api.ai71.ai/v1"
    _key_variable = "AI71_API_KEY"


class AI71(_ClientMixin, client.Client):
    pass


class AsyncAI71(_ClientMixin, client.AsyncClient):
    pass



from typing import Literal, Sequence

import pydantic

from ..types.chat_completion_message import (
    ChatCompletionMessage,
    ChatCompletionMessageParam,
)
from ..types.completion import CommonChoice, CommonCompletion, CommonCreateCompletion


class _CreateChatCompletion(pydantic.BaseModel):
    messages: Sequence[ChatCompletionMessageParam] = pydantic.Field(
        description="A sequence of chat messages constituting the current dialog.",
    )


class CreateChatCompletion(CommonCreateCompletion, _CreateChatCompletion):
    pass


class Choice(CommonChoice):
    message: ChatCompletionMessage = pydantic.Field(
        description="Generated chat response.",
    )


class ChatCompletion(CommonCompletion):
    choices: Sequence[Choice] = pydantic.Field(
        default="A list of generated chat responses."
    )
    object: Literal["chat.completion"] = pydantic.Field(
        description="Constant `chat.completion`"
    )



from typing import List, Literal, Optional, Sequence

import annotated_types
import pydantic
from typing_extensions import Annotated

from ..types.completion_usage import CompletionUsage
from ..types.finish_reason import FinishReason
from ..types.model import Model


class CommonCreateCompletion(pydantic.BaseModel):
    model: Model = pydantic.Field(description="The identifier of the model to use.")
    stream: bool = pydantic.Field(
        default=False,
        description="Enables sending tokens via streaming mode using server-sent events.",
    )
    max_tokens: Optional[int] = pydantic.Field(
        default=None,
        description="The upper limit on the number of tokens in the generated output.",
    )
    stop: Annotated[List[str], annotated_types.Len(max_length=8)] = pydantic.Field(
        default=[],
        description="A list of up to 8 tokens that, if encountered, will terminate the text generation.",
    )
    temperature: pydantic.NonNegativeFloat = pydantic.Field(
        default=0.0,
        description="Adjusts the probability distribution of token selection using a temperature. Higher values favor less predictable, diverse outputs, while lower values prioritize deterministic choices.",
    )
    frequency_penalty: float = pydantic.Field(
        default=0.0,
        description="The higher this value, the more it encourages variety by making the model less likely to repeat frequently used words or phrases.",
    )
    presence_penalty: float = pydantic.Field(
        default=0.0,
        description="The higher this value, the more it encourages variety by making the model less likely to repeat words or phrases which were already used (regardless of how frequently they were used).",
    )
    top_p: float = pydantic.Field(
        default=1.0,
        gt=0.0,
        le=1.0,
        description="Enables nucleus sampling by restricting the model to tokens within the cumulative probability mass defined by `top_p`.",
    )
    top_k: Optional[pydantic.PositiveInt] = pydantic.Field(
        default=None,
        description="Restricts token selection to the `top_k` most probable candidates according to the model predictions.",
    )


class _CreateCompletion(pydantic.BaseModel):
    prompt: str = pydantic.Field(
        description="Defines the initial context for auto regressive text generation.",
    )


class CreateCompletion(CommonCreateCompletion, _CreateCompletion):
    pass


class CommonChoice(pydantic.BaseModel):
    finish_reason: FinishReason = pydantic.Field(
        description="Specifies the reason for the token generation termination, indicating either a natural stopping point `stop` or a reach of a maximum token count constraint.",
    )
    index: int = pydantic.Field(
        description="The position of the choice in the choices array.",
    )


class Choice(CommonChoice):
    text: str = pydantic.Field(
        description="Generated text response.",
    )


class CommonCompletion(pydantic.BaseModel):
    id: str = pydantic.Field(
        description="A unique identifier that serves as a reference for the generated completion."
    )
    created: int = pydantic.Field(
        description="A Unix timestamp, indicating the time of completion creation."
    )
    model: str = pydantic.Field(
        description="Identifier of the model used to create the completion."
    )
    usage: Optional[CompletionUsage] = pydantic.Field(
        description="Identifier of the model used to create the completion."
    )


class Completion(CommonCompletion):
    choices: Sequence[Choice] = pydantic.Field(
        default="A list of generated text responses."
    )
    object: Literal["completion"] = pydantic.Field(description="Constant `completion`")



import pydantic


class CompletionUsage(pydantic.BaseModel):
    completion_tokens: int = pydantic.Field(
        description="Number of tokens in the generated response.",
    )
    prompt_tokens: int = pydantic.Field(
        description="Number of tokens in the generated response.",
    )
    total_tokens: int = pydantic.Field(
        description="Total number of tokens in the request and response.",
    )



import pydantic
from typing_extensions import Annotated

_UUID_PATTERN = (
    r"[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}"
)
_MODELS = [
    "tiiuae/falcon-180B-chat",
    "tiiuae/falcon-40B",
    "tiiuae/falcon-40B-instruct",
    "tiiuae/falcon-7B",
    "tiiuae/falcon-7B-instruct",
    "tiiuae/falcon-11B-vlm",
    "tiiuae/falcon-11B",
    "tiiuae/falcon-mamba-7B-instruct",
]
_FT_MODELS = (
    model.replace("tiiuae/", "").replace("B", b)
    for model in _MODELS
    for b in ("B", "b")
)
FINETUNE_MODEL_PATTERN = rf"ft:(?P<model>{'|'.join(_FT_MODELS)}):(?<org_id>{_UUID_PATTERN})(?::(?P<suffix>[^:]*))?:(?<uuid>{_UUID_PATTERN})"
_BASE_MODELS = (
    model.replace("/", r"\/").replace("B", b) for model in _MODELS for b in ("B", "b")
)
BASE_MODEL_PATTERN = rf"{'|'.join(_BASE_MODELS)}"
MODEL_PATTERN = rf"^(?:{BASE_MODEL_PATTERN})|(?:{FINETUNE_MODEL_PATTERN})$"
Model = Annotated[str, pydantic.StringConstraints(to_lower=True, pattern=MODEL_PATTERN)]


import os
from typing import Optional

import httpx

from .. import exceptions
from ..resources import chat, completions
from . import base_client

# Default timeout is 10 minutes.
DEFAULT_TIMEOUT = httpx.Timeout(600.0)
DEFAULT_URL = httpx.URL("https://api.ai71.ai/v1")


class _ClientMixin:
    _key_variable: str

    def _make_status_error(self, response: httpx.Response) -> exceptions.APIError:
        return exceptions.APIError()


class Client(_ClientMixin, base_client.BaseClient):
    def __init__(
        self,
        api_key: Optional[str] = None,
        timeout: httpx.Timeout = DEFAULT_TIMEOUT,
        base_url: httpx.URL = DEFAULT_URL,
    ) -> None:
        if api_key is None:
            api_key = os.getenv(self._key_variable)

        self.chat = chat.Chat(self)
        self.completions = completions.Completions(self)

        super().__init__(
            headers={
                "Authorization": f"Bearer {api_key}",
            },
            base_url=base_url,
            timeout=timeout,
        )


class AsyncClient(_ClientMixin, base_client.AsyncBaseClient):
    def __init__(
        self,
        api_key: Optional[str] = None,
        timeout: httpx.Timeout = DEFAULT_TIMEOUT,
        base_url: httpx.URL = DEFAULT_URL,
    ) -> None:
        if api_key is None:
            api_key = os.getenv(self._key_variable)

        self.chat = chat.AsyncChat(self)
        self.completion = completions.AsyncCompletions(self)

        super().__init__(
            headers={
                "Authorization": f"Bearer {api_key}",
            },
            base_url=base_url,
            timeout=timeout,
        )
